/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb;

import javax.jbi.JBIException;
import org.ow2.petals.bc.ejb.service.EjbService;
import org.ow2.petals.bc.ejb.su.EjbBcServiceUnitListener;
import org.ow2.petals.component.framework.bc.AbstractBindingComponent;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;

/**
 * An EJB binding component. This component allows the user to call an EJB from
 * the PEtALS ESB, using the configuration given in the service unit loaded by
 * the EjbBcServiceUnitListener class.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbComponent extends AbstractBindingComponent {

    /**
     * The EJB connection pool.
     */
    private EjbConnectionPool ejbConnectionPool;

    /**
     * The EJB service class.
     */
    private EjbService ejbService;

    /**
     * Initialize the component. Creates a new connection pool, a new instance
     * of the EjbService class, and binds the right SUListener to the component.
     *
     * @throws javax.jbi.JBIException
     *      if any problem occurs while performing the initialization
     */
    @Override
    protected void doInit() throws JBIException {
        super.doInit();

        //Initialize the connection pool
        this.ejbConnectionPool = new EjbConnectionPool(this.getLogger());

        //Initialize the EJB service
        this.ejbService = new EjbService(this.getLogger(), this);

        //Binds the component and to its service unit listener
        //final EjbBcServiceUnitListener serviceUnitListener = new EjbBcServiceUnitListener(this.getLogger(), this);
        //this.setServiceUnitListener(serviceUnitListener);
    }

    /**
     * Accessor on the connection pool associated to the EJB component.
     *
     * @return a reference onto the EJB pool
     */
    public EjbConnectionPool getEjbConnectionPool() {
        return this.ejbConnectionPool;
    }
    
    @Override
    protected AbstractServiceUnitManager createServiceUnitManager() {
        return new EjbBcServiceUnitListener(this.getLogger(), this);
    }

    /**
     * Accessor on the service class of this component.
     *
     * @return the service class of this component
     */
    public EjbService getEjbService() {
        return this.ejbService;
    }
}
