/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.configuration;

import java.util.ArrayList;
import java.util.List;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * Holds the informations relative to the security artifacts required to
 * have an access on the app server.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class SecurityElement implements ConfigurationElement {

    public static final String CONFIGURATION_ELEMENT_NAME = "security";

    /**
     * Name of the security class used to provide a connection to the app
     * server.
     */
    private String securityName;

    /**
     * kinda username.
     */
    private String securityPrincipal;

    /**
     * Kinda password.
     */
    private String securityCredentials;

    /**
     * Default constructor.
     */
    public SecurityElement() {

    }

    /**
     * Default constructor.
     *
     * @param securityName
     *      security name class
     * @param securityPrincipal
     *      security principal (username)
     * @param securityCredentials
     *      security credentials (password)
     */
    public SecurityElement(final String securityName,
            final String securityPrincipal,
            final String securityCredentials) {

        //Sets the fields
        this.securityName = securityName;
        this.securityPrincipal = securityPrincipal;
        this.securityCredentials = securityCredentials;
    }

    /**
     * Accessor on the security name used for authentication on the app server.
     *
     * @return the security name
     */
    public final String getSecurityName() {
        return securityName;
    }

    /**
     * Setter for the security name used for the authentication on the app server.
     *
     * @param securityName the security name
     */
    public final void setSecurityName(final String securityName) {
        this.securityName = securityName;
    }

    /**
     * Accessor on the security principal (username).
     *
     * @return the security principal
     */
    public final String getSecurityPrincipal() {
        return securityPrincipal;
    }

    /**
     * Setter for the security principal (username).
     *
     * @param securityPrincipal the security principal (username)
     */
    public final void setSecurityPrincipal(final String securityPrincipal) {
        this.securityPrincipal = securityPrincipal;
    }

    /**
     * Accessor on the security credentials (password).
     *
     * @return the security credentials
     */
    public final String getSecurityCredentials() {
        return securityCredentials;
    }

    /**
     * Setter for the security credentials (password).
     *
     * @param securityCredentials the security credentials
     */
    public final void setSecurityCredentials(final String securityCredentials) {
        this.securityCredentials = securityCredentials;
    }

    /**
     * Retrieve all the missing configuration parameters for the current
     * configuration element. The missing configuration elements are returned
     * as a List.
     *
     * @return a list of the missing elements
     */
    public List<String> findMissingElements() {
        List<String> missingElements = new ArrayList<String>();

        return missingElements;
    }

    /**
     * Retrieve all the invalid configuration parameters for the current
     * configuration element. The invalid configuration elements are returned
     * as a List.
     *
     * @return a list of the invalid elements
     */
    public List<String> findInvalidElements() {
        List<String> invalidElements = new ArrayList<String>();

        return invalidElements;
    }

    /**
     * Return a String description of the current instance.
     *
     * @return
     *      a String description of the current instance
     */
    @Override
    public String toString() {
        StringBuilder toStringSB = new StringBuilder();
        toStringSB.append("securityName=" + this.securityName);
        toStringSB.append(", ");
        toStringSB.append("securityPrincipal=" + this.securityPrincipal);
        toStringSB.append(", ");
        toStringSB.append("securityCredentials(hashcode)=" + this.securityCredentials.hashCode());

        return toStringSB.toString();
    }

    /**
     * Checks if the field SECURITY_NAME is not null or empty.
     *
     * @return true if the field is not null or empty
     */
    private final boolean hasSecurityName() {

        if (StringHelper.isNullOrEmpty(this.securityName)) {
            return false;
        }

        return true;
    }

    /**
     * Checks if the field SECURITY_PRINCIPAL is not null or empty.
     *
     * @return true if the field is not null or empty
     */
    private final boolean hasSecurityPrincipal() {

        if (StringHelper.isNullOrEmpty(this.securityPrincipal)) {
            return false;
        }

        return true;
    }

    /**
     * Checks if the field SECURITY_CREDENTIALS is not null or empty.
     *
     * @return true if the field is not null or empty
     */
    private final boolean hasSecurityCredentials() {

        if (StringHelper.isNullOrEmpty(this.securityCredentials)) {
            return false;
        }

        return true;
    }
}
