/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.listeners;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import javax.jbi.JBIException;

import org.ow2.petals.bc.ejb.EjbComponent;
import org.ow2.petals.bc.ejb.service.EjbService;


/**
 * JBIListener of the BC-EJB component. Reacts on each call onto the component
 * and call the appropriate methods from the service class.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbBcJbiListener extends AbstractJBIListener {

    /**
     * Reacts on each call onto the EJB component and call the appropriate
     * methods from the service class.
     *
     * @param exchange
     *      the received message
     */
    @Override
    public boolean onJBIMessage(Exchange exchange) {
        //If the received exchange is null
        if (exchange == null) {
            this.getLogger().severe("Receive a null message exchange");
            return true;
        }

        //Ignores the ACK messages
        if ( ! (exchange.isActiveStatus() && exchange.getFault() == null)) {
            this.getLogger().finest("Receive an ack or a Fault, skip it");
            return true;
        }

        //Try to call the EJB
        try {

            //Retrieve the service
            EjbService service = ((EjbComponent) getComponent()).getEjbService();

            //Calls the EJB
            service.callEjb(getProvides(), exchange);

        } catch (JBIException ex) {
            this.getLogger().severe(ex.getMessage());

            if (exchange.isInOnlyPattern()) {
                exchange.setError(ex);
            }
            else if (exchange.isInOutPattern()) {
                exchange.setError(ex);
            }
            else if (exchange.isRobustInOnlyPattern()) {
                exchange.setError(ex);
            }
            else if (exchange.isInOptionalOutPattern()) {
                exchange.setError(ex);
            }
            else {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Unable to propagate the exception : ");
                errorSB.append(ex.getMessage());
                errorSB.append(": Invalid MEP");
                this.getLogger().severe(errorSB.toString());
                return true;
            }
        }
        
        return true;
    }

}
