/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.ejb.su;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.management.DeploymentException;

import org.ow2.petals.bc.ejb.EjbComponent;
import org.ow2.petals.bc.ejb.EjbConnectionPoolElement;
import org.ow2.petals.bc.ejb.configuration.EjbConfiguration;
import org.ow2.petals.bc.ejb.configuration.EjbConfigurationBuilder;
import org.ow2.petals.bc.ejb.exceptions.ConfigurationException;
import org.ow2.petals.bc.ejb.exceptions.ConnectionException;
import org.ow2.petals.bc.ejb.exceptions.EjbConnectionPoolException;
import org.ow2.petals.bc.ejb.service.EjbConnection;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.BindingComponentServiceUnitManager;
import org.ow2.petals.component.framework.util.ClassLoaderUtil;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.databinding.jaxb.DataBindingException;
import org.ow2.petals.databinding.jaxb.ServiceFactory;
import org.ow2.petals.databinding.jaxb.service.Service;
import org.ow2.petals.databinding.jaxb.tools.JaxbClassGenerator;

/**
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class EjbBcServiceUnitListener extends BindingComponentServiceUnitManager {

    /**
     * A reference to the EJB component.
     */
    private EjbComponent component;

    //TODO : documentation
    private Logger logger;

    //TODO : documentation
    public EjbBcServiceUnitListener(final Logger logger, final EjbComponent ejbComponent) {
        super(ejbComponent);
        this.logger = logger;
        this.component = ejbComponent;
    }

    private String getSURootPath(String serviceUnitName) {
        return getServiceUnitDataHandlers().get(serviceUnitName).getInstallRoot();
    }
    
    private Jbi getSUDescriptor(String serviceUnitName) {
        return getServiceUnitDataHandlers().get(serviceUnitName).getDescriptor();
    }
    
    @Override
    public void start(String serviceUnitName) throws DeploymentException {
        super.start(serviceUnitName);
        //Retrieve all the provides given in the SU
        List<Provides> providesList = getSUDescriptor(serviceUnitName).getServices().getProvides();

        //Process all the provides given in the SU
        for (final Provides provides : providesList) {

            EjbConfiguration ejbConfiguration = null;
            try {

                //Loads the ConfigurationExtensions using the given property file
                ConfigurationExtensions extensions = null;
                if ((this.component.getComponentConfiguration().getPropertiesFile() != null)
                        && (!StringHelper.isNullOrEmpty(this.component.getComponentConfiguration().getPropertiesFile().getValue()))) {

                    extensions = new ConfigurationExtensions(provides.getAny(),
                            this.component.getComponentConfiguration().getPropertiesFile().getValue());

                } else {
                    extensions = new ConfigurationExtensions(provides.getAny());
                }

                //Retrieve the WSDL
                final String wsdlFileName = provides.getWsdl();

                //Retrieve the ServiceName
                final String serviceName = provides.getServiceName().getLocalPart();

                //Checks if the provides list is not empty
                if (provides.getAny().size() == 0) {
                    StringBuilder errorSB = new StringBuilder();
                    errorSB.append("Can't create ConfigurationExtensions from the given ");
                    errorSB.append("JBI descriptor content: ");
                    errorSB.append(" no provides configuration found.");
                    throw new ConfigurationException(errorSB.toString());
                }

                //Retrieve the configuration for ONE provide
                ejbConfiguration = EjbConfigurationBuilder.buildEjbConfiguration(extensions, wsdlFileName, serviceName);

                //Registers the EJB
                this.registerEjb(provides, ejbConfiguration, getSURootPath(serviceUnitName));

            } catch (EjbConnectionPoolException ex) {
                throw new DeploymentException("Can't deploy the service unit: ", ex);
            } catch (ConnectionException ex) {
                throw new DeploymentException("Can't deploy the service unit: ", ex);
            } catch (ConfigurationException ex) {
                throw new DeploymentException("Can't deploy the service unit: ", ex);
            } catch(PEtALSCDKException ex) {
                throw new DeploymentException("Can't deploy the service unit: ", ex);
            }
        }
    }

    @Override
    public void stop(String serviceUnitName) throws DeploymentException {
        super.stop(serviceUnitName);
        //Retrieve all the provides given in the SU
        Jbi descriptor = getSUDescriptor(serviceUnitName);
        if(descriptor != null) { // Best effort strategy (Can't remove ? keep on playing)
            List<Provides> providesList = descriptor.getServices().getProvides();

            for (Provides provides : providesList) {

                try {

                    //Removes the EJB
                    this.component.getEjbConnectionPool().removeEjb(provides);

                } catch (EjbConnectionPoolException ex) {
                    throw new DeploymentException("Can't undeploy the service unit: ", ex);
                }
            }
        }

    }

    //TODO : documentation
    private void registerEjb(
            final Provides provides,
            final EjbConfiguration configuration,
            final String suRootPath)
            throws ConnectionException, PEtALSCDKException, EjbConnectionPoolException, ConfigurationException {

        //Creates a new suClassLoader
        ClassLoader suClassLoader = this.createSUClassLoader(suRootPath);

        //Backups the current context ClassLoader
        ClassLoader oldContextClassLoader = Thread.currentThread().getContextClassLoader();

        //Change the current context ClassLoader to the suClassLoader
        Thread.currentThread().setContextClassLoader(suClassLoader);

        try {
            //Creates a new EjbConnectionObject
            EjbConnection ejbConnection = new EjbConnection(configuration, suClassLoader, this.component.getLogger());

            //Creates a new Initial Context
            //Context initialContext = ejbConnection.createInitialContext();

            //Retrieve the EJB matching the given name
            Object ejb = ejbConnection.retrieveObject(configuration);

            //Creates the databinding
            Service databindingService = this.createJAXBDatabinding(configuration, suClassLoader, ejb.getClass(), suRootPath);

            //Create the value to be added to the pool
            EjbConnectionPoolElement poolElement = new EjbConnectionPoolElement(configuration, ejb, databindingService, suClassLoader);

            //Register the EJB
            this.component.getEjbConnectionPool().addEjb(provides, poolElement);
        }
        finally {
            //Reset the context classloader
            Thread.currentThread().setContextClassLoader(oldContextClassLoader);
        }

    }

    //TODO : documentation
    private Service createJAXBDatabinding(
            final EjbConfiguration configuration,
            final ClassLoader suClassLoader,
            final Class<?> ejbServiceClass,
            final String suRootPath) throws PEtALSCDKException, ConfigurationException {

        //Saves the system properties
        final Properties properties = (Properties) System.getProperties().clone();

        //To hold the databinding service
        Service databindingService = null;

        try {
            //Gets the WSDL filename
            final String wsdlFileName = configuration.getDatabindingElement().getWsdlFileName();

            //Gets the service name
            final String serviceName = configuration.getDatabindingElement().getServiceName();

            //Gets the component's ClassPath
            URL[] componentClassPath = ClassLoaderUtil.getUrls(this.component.getContext().getInstallRoot());
            List<String> classPath = new ArrayList<String>();
            for (URL url : componentClassPath) {
                classPath.add(url.getPath());
            }

            //Call JAXB to generate the Databinding classes for the EJB interface
            final String wsdlFilePath = suRootPath + File.separator + wsdlFileName;
            final String jaxbBaseDir = suRootPath + serviceName + "-jaxb";
            (new File(jaxbBaseDir)).mkdirs();
            JaxbClassGenerator.generateJaxbClass(wsdlFilePath, jaxbBaseDir, serviceName);

            //Compiles the generated JAXB classes
            JaxbClassGenerator.compileJaxbClass(jaxbBaseDir, serviceName, classPath);

            //Creates the service object to Marshall/Unmarshall the request / response
            //message parameters and marshall method results
            final String jaxbClassesUrl = "file://" + jaxbBaseDir + "/target/classes/";
            ServiceFactory serviceFactory = ServiceFactory.getInstance();

            databindingService = serviceFactory.crateService(wsdlFilePath, jaxbClassesUrl, suClassLoader, ejbServiceClass);
        } catch (DataBindingException ex) {
            StringBuilder errorSB = new StringBuilder("Databinding error : ");
            errorSB.append("can't create the databinding service : ");
            this.logger.log(Level.SEVERE, errorSB.toString(), ex);
            throw new PEtALSCDKException(errorSB.toString(), ex);
        } finally {
            //Reset the system properties
            System.setProperties(properties);
        }
        //At this point :
        // - the databinding service seem to be OK
        // - the "normal" classpath has been restored

        return databindingService;
    }

    /**
     * Creates a new ClassLoader used only for that SU.
     *
     * @param suRootPath
     *      the SU root path
     * @return
     *      a new ClassLoader (its parent's ClassLoader is the one used to load
     *      the current class)
     */
    private ClassLoader createSUClassLoader(String suRootPath) {
        return ClassLoaderUtil.createClassLoader(suRootPath, this.getClass().getClassLoader());
    }
}
